import React, { useContext, useEffect, useState } from 'react';
import { Headline, CommonMenuList, LoadingCircle, ContentInfo, NavList, NavItem, TagChip } from '../component/mod';

import { Box, Typography, Chip, Pagination, Button } from '@mui/material';
import ContentAccessor, { QueryListOption, Document } from '../accessor/document';
import { toQueryString } from '../accessor/util';

import { useLocation } from 'react-router-dom';
import { QueryStringToMap } from '../accessor/util';



export type GalleryProp = {
    option?: QueryListOption;
    diff: string;
};
type GalleryState = {
    documents: Document[] | undefined;
}

export const GalleryInfo = (props: GalleryProp) => {
    const [state, setState] = useState<GalleryState>({ documents: undefined });
    const [error, setError] = useState<string | null>(null);
    const [loadAll, setLoadAll] = useState(false);
    useEffect(() => {
        const abortController = new AbortController();
        console.log('load first',props.option);
        const load = (async () => {
            try{
                const c = await ContentAccessor.findList(props.option);
                //todo : if c is undefined, retry to fetch 3 times. and show error message.
                setState({ documents: c });
                setLoadAll(c.length == 0);
            }
            catch(e){
                if(e instanceof Error){
                    setError(e.message);
                }
                else{
                    setError("unknown error");
                }
            }
        });
        load();
    }, [props.diff]);
    const queryString = toQueryString(props.option ?? {});
    if (state.documents === undefined && error == null) {
        return (<LoadingCircle />);
    }
    else {
        return (
            <Box sx={{
                display: 'grid',
                gridRowGap: '1rem'
            }}>
                {props.option !== undefined && props.diff !== "" && <Box>
                    <Typography variant="h6">search for</Typography>
                    {props.option.word !== undefined && <Chip label={"search : " + decodeURIComponent(props.option.word)}></Chip>}
                    {props.option.content_type !== undefined && <Chip label={"type : " + props.option.content_type}></Chip>}
                    {props.option.allow_tag !== undefined && props.option.allow_tag.map(x => (
                        <TagChip key={x} tagname={decodeURIComponent(x)} label={decodeURIComponent(x)}></TagChip>))}
                </Box>}
                {
                    state.documents && state.documents.map(x => {
                        return (<ContentInfo document={x} key={x.id}
                            gallery={`/search?${queryString}`} short />);
                    })
                }
                {(error && <Typography variant="h5">Error : {error}</Typography>)}
                <Typography variant="body1" sx={{
                    justifyContent: "center",
                    textAlign:"center"
                }}>{state.documents ? state.documents.length : "null"} loaded...</Typography>
                <Button onClick={()=>loadMore()} disabled={loadAll}>{loadAll ? "Load All" : "Load More"}</Button>
            </Box>
        );
        function loadMore() {
            let option = {...props.option};
            if(state.documents === undefined || state.documents.length === 0){
                console.log("loadall");
                setLoadAll(true);
                return;
            }
            const prev_documents = state.documents;
            option.cursor = prev_documents[prev_documents.length - 1].id;
            console.log("load more", option);
            const load = (async () => {
                const c = await ContentAccessor.findList(option);
                if (c.length === 0) {
                    setLoadAll(true);
                }
                else{
                    setState({ documents: [...prev_documents, ...c] });
                }
            });
            load();
        }
    }
}

export const Gallery = () => {
    const location = useLocation();
    const query = QueryStringToMap(location.search);
    const menu_list = CommonMenuList({ url: location.search });
    let option: QueryListOption = query;
    option.allow_tag = typeof option.allow_tag === "string" ? [option.allow_tag] : option.allow_tag;
    option.limit = typeof query['limit'] === "string" ? parseInt(query['limit']) : undefined;
    return (<Headline menu={menu_list}>
        <GalleryInfo diff={location.search} option={query}></GalleryInfo>
    </Headline>)
}